# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from apt_forktracer.testlib.fake_package_file import FakePackageFile

class FakeVersion(object):
	"""Pretends to be the pkgCache::Version object from apt_pkg. We cannot use
	the real one, because it is tied to the binary cache, which is difficult to
	construct and control."""
	def __init__(self, version_string = '1.2.3'):
		self.ver_str = version_string
		self.file_list = []
	def append_package_file(self, package_file):
		tuple = (package_file,1)
		self.file_list.append(tuple)
	def __str__(self):
		files = ''
		for f in self.file_list:
			files += '(%s,%d),' % (str(f[0]), f[1])
		return '<FakeVersion %s [%s]>' % (self.ver_str, files)
	def _create(string, origins):
		"""Factory method."""
		fv = FakeVersion(string)
		for o in origins:
			if o == 'dpkg':
				fv.append_package_file(FakePackageFile(type = 'dpkg'))
			else:
				fv.append_package_file(FakePackageFile(origin = o))
		return fv
	_create = staticmethod(_create)




#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008-2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.facter import Facter

class TestFacterWithoutArgs(unittest.TestCase):
	def setUp(self):
		self.facter = Facter()
	def testDistributorsIdReturnsSomething(self):
		self.assert_(self.facter.distributors_id)

class TestFacterWithLSBReleaseModuleArg(unittest.TestCase):
	def setUp(self):
		self.facter = Facter(lsb_release_module = 'apt_forktracer.testlib.fake_lsb_release')
	def testDistributorsIdReturnsFakeDistribursId(self):
		self.assertEquals(self.facter.distributors_id, 'fake_distributors_id')

class TestFacterWithLSBReleaseArg(unittest.TestCase):
	def setUp(self):
		self.facter = Facter(lsb_release_module = 'non.existant', lsb_release = 'echo Foobar')
	def testDistributorsIdReturnsFoobar(self):
		self.assertEquals(self.facter.distributors_id, 'Foobar')

class TestFacterWithBrokenLSBReleaseArgAndValidFileArg(unittest.TestCase):
	def setUp(self):
		self.facter = Facter(lsb_release_module = 'non.existant', lsb_release = 'false', file = 'test-data/lsb-release')
	def testDistributorsIdReturnsUbuntu(self):
		self.assertEquals(self.facter.distributors_id, 'Ubuntu-in-test-data')

class TestFacterReadFirstWord(unittest.TestCase):
	def setUp(self):
		self.facter = Facter()
	def testUsualIssueFile(self):
		self.assertEquals(self.facter.get_distrib_id_from_file('test-data/lsb-release'), 'Ubuntu-in-test-data')
	def testEmptyFile(self):
		self.assertEquals(self.facter.get_distrib_id_from_file('test-data/empty'), None)
	def testMissingFile(self):
		self.assertEquals(self.facter.get_distrib_id_from_file('test-data/does_not_exist'), None)

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# Runs all tests.
#
# This script must be run from the directory it is in.
# Individual tests may also be executed directly like: ./tests/test_whatever.py
from unittest import TestLoader, TextTestRunner
import sys
sys.path.append('lib')

# The following is ugly, but it's the only method I've found of loading all
# tests from all subpackages of package 'tests', without hardcoding their names
# here.

# We have to import both tests and its subpackages, otherwise dir() does not
# find subpackages.
from apt_forktracer import tests
from apt_forktracer.tests import *

def run_tests(verbosity):
	# Filter out special attributes of the package, to get only subpackages.
	test_names = [n for n in dir(tests) if not n.startswith('__')]
	# Load and run all tests
	all_tests = TestLoader().loadTestsFromNames(test_names, tests)
	TextTestRunner(verbosity=verbosity).run(all_tests)

if __name__ == '__main__':
	if len(sys.argv) == 2:
		run_tests(int(sys.argv[1]))
	else:
		run_tests(1)



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.version_adapter import VersionAdapter

class TestBaseVersionAdapter(test_helper.MoxTestCase):
	def setUp(self):
		super(TestBaseVersionAdapter, self).setUp()
		self.setUpFakeVersion()
		self.setUpVersionAdapter()
	def setUpFakeVersion(self):
		self.fake_version = FakeVersion()
	def setUpVersionAdapter(self):
		self.va = VersionAdapter(self.fake_version)
	def testString(self):
		self.assertEquals(self.va.string, '1.2.3')
	def testLenOfFilesListPositive(self):
		"If there is a version, it must come from at least one file (dpkg status)."
		self.assert_(self.va.files > 0)
	def testBasicStringification(self):
		self.assertMatches(str(self.va), r'<VersionAdapter 1.2.3')
	def setUpAddAFile(self):
		self.fake_version.append_package_file(FakePackageFile(path = '/b/lah'))

class TestVersionOfficiallyAvailable(test_helper.MoxTestCase):
	"""Testing the is_officially_available() method."""
	def setUp(self):
		super(TestVersionOfficiallyAvailable, self).setUp()
		self.mock_apt_version = self.struct()
		self.mock_apt_version.ver_str = '1.2.3'
		self.mock_apt_version.file_list = [(FakePackageFile(type = 'dpkg'), 1)]
		self.mock_facter = self._create_mock_facter('Debian')
	def assertVersionIsOfficiallyAvailable(self):
		self.assert_(VersionAdapter(self.mock_apt_version).is_officially_available(self.mock_facter))
	def assertVersionIsNotOfficiallyAvailable(self):
		self.assert_(not VersionAdapter(self.mock_apt_version).is_officially_available(self.mock_facter))
	def testVersionNotAvailableApartFromInstalled(self):
		self.assertVersionIsNotOfficiallyAvailable()
	def testVersionAvailableFromUnofficialPackageFile(self):
		self.mock_apt_version.file_list.append((FakePackageFile(type = 'normal', origin = 'Unofficial'), 2))
		self.assertVersionIsNotOfficiallyAvailable()
	def testVersionAvailableFromOfficialPackageFile(self):
		self.mock_apt_version.file_list.append((FakePackageFile(type = 'normal', origin = 'Debian'), 2))
		self.assertVersionIsOfficiallyAvailable()
	def testVersionAvailableFromBothOfficialAndUnofficialPackageFiles(self):
		self.mock_apt_version.file_list.append((FakePackageFile(type = 'normal', origin = 'Debian'), 2))
		self.mock_apt_version.file_list.append((FakePackageFile(type = 'normal', origin = 'Unofficial'), 1))
		self.assertVersionIsOfficiallyAvailable()

class TestZeroFileVersionAdapter(TestBaseVersionAdapter):
	def testFileCount(self):
		self.assertEquals(len(self.va.files), 0)
	def testStringification(self):
		self.assertMatches(str(self.va), r'<VersionAdapter 1.2.3 \[\]>')

class TestOneFileVersionAdapter(TestBaseVersionAdapter):
	def setUp(self):
		super(TestOneFileVersionAdapter, self).setUp()
		self.setUpFakeVersion()
		self.setUpAddAFile()
		self.setUpVersionAdapter()
	def testFileCount(self):
		self.assertEquals(len(self.va.files), 1)
		self.assertEquals(self.va.files[0].name, '/b/lah')
	def testStringification(self):
		self.assertMatches(str(self.va), r'<VersionAdapter 1.2.3 \[.*/b/lah.*\]>')

class TestThreeFilesVersionAdapter(TestBaseVersionAdapter):
	def setUp(self):
		super(TestThreeFilesVersionAdapter, self).setUp()
		self.setUpFakeVersion()
		for i in range(3):
			self.setUpAddAFile()
		self.setUpVersionAdapter()
	def testOneFile(self):
		self.assertEquals(len(self.va.files), 3)
	def testStringification(self):
		self.assertMatches(str(self.va), r'<VersionAdapter 1.2.3 \[.*/b/lah.*/b/lah.*/b/lah.*\]>')

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class Reporter:
	"""Reports a status to the user. Currently just pretty-prints it to standard output. Might syslog or whatever..."""
	def report(self, status):
		"""Prints a formatted report for a given status to standard output."""
		print self.format(status)
	def format(self, status):
		"""Returns a formatted report for a given status."""
		if (not status.installed_version) and (not status.candidate_version):
			pending_version_info = ''
		elif not status.installed_version:
			pending_version_info = '->%s' % status.candidate_version.string
		elif not status.candidate_version:
			pending_version_info = '%s->' % status.installed_version.string
		elif status.installed_version.string == status.candidate_version.string:
			pending_version_info = status.installed_version.string
		else:
			pending_version_info = '%s->%s' % (status.installed_version.string, status.candidate_version.string)
		version_info = ''
		for origin,versions in status.versions_by_origin.items():
			version_info += ' [%s: %s]' % (origin, ' '.join([v.string for v in versions]))
		return '%s (%s)' % (status.package_name, pending_version_info) + version_info




# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from apt_forktracer.version_adapter import VersionAdapter

class DepCacheAdapterFactory:
	"""Factory for DepCacheAdapter objects."""
	def create_depcache_adapter(self, apt_dep_cache):
		return DepCacheAdapter(apt_dep_cache)

class DepCacheAdapter:
	"""Wrapper for the pkgDepCache class. See the AptPkgAdapter for why this
	class is necessary."""
	def __init__(self, apt_dep_cache):
		self.apt_dep_cache = apt_dep_cache
	def get_candidate_version(self, pkg):
		"""Returns a VersionAdapter object representing the candidate version
		of the given PackageAdapter object pkg. Returns None if there is no
		candidate version."""
		apt_version = self.apt_dep_cache.get_candidate_ver(pkg.apt_package)
		if not apt_version:
			return None
		else:
			return VersionAdapter(apt_version)



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class Config:
	"""Encapsulates config file information."""
	def __init__(self):
		self._stanzas = []
		self._package_map = {}

	def add(self, stanza):
		"""Adds a given stanza for later retrieval."""
		self._stanzas.append(stanza)
		name = stanza.get('package')
		if self._package_map.has_key(name):
			self._package_map[name].append(stanza)
		else:
			self._package_map[name] = [ stanza ]

	def package(self, package_name):
		"""Returns a (potentially empty) list of all stanzas for the given
		package name."""
		if self._package_map.has_key(package_name):
			return self._package_map[package_name]
		else:
			return []



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import apt_pkg

class FakePackage:
	"""Pretends to be the pkgCache::Package object from apt_pkg. We cannot use
	the real one, because it is tied to the binary cache, which is difficult to
	construct and control."""
	def __init__(self, current_state = apt_pkg.CURSTATE_INSTALLED, name = 'afake'):
		self.name = name
		self.version_list = []
		self.current_state = current_state
		self.current_ver = None
	def append_version(self, version, current = False):
		self.version_list.append(version)
		if current:
			self.current_ver = version
	def __str__(self):
		vers = ''
		for v in self.version_list:
			vers += str(v) + ','
		return '<FakePackage(%s) %s v=%s [%s]>' % (self.current_state, self.name, str(self.current_ver), vers)



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from apt_forktracer.version_adapter import VersionAdapter
from apt_forktracer.status import Status

class PackageAdapterFactory:
	"""Factory for the PackageAdapter objects."""
	def __init__(self, depcache_adapter = None):
		"""The depcache_adapter argument is an optional DepCacheAdapter object."""
		self.depcache_adapter = depcache_adapter

	def create_package_adapter(self, apt_package):
		"""Instantiates a new PackageAdapter from the given apt_package object.
		Sets the candidate_version attribute on the created object, if a
		DepCacheAdapter was provided when creating the factory.
		"""
		pa = PackageAdapter(apt_package)
		if self.depcache_adapter:
			pa.candidate_version = self.depcache_adapter.get_candidate_version(pa)
		return pa

class PackageAdapter:
	"""Wrapper for the pkgCache::Package class. See AptPkgAdapter for why it is
	necessary.
	"""
	def __init__(self, apt_package):
		self.apt_package = apt_package
		self.name = apt_package.name
		if apt_package.current_ver:
			self.current_version = VersionAdapter(apt_package.current_ver)
		else:
			self.current_version = None
		self.candidate_version = None
		self.versions = [VersionAdapter(v) for v in apt_package.version_list]

	def get_status(self, facter):
		"""Creates a new Status object based on this one, given a Facter."""
		versions_by_origin = {}
		for version in self.versions:
			for pfa in version.files:
				origin = pfa.origin
				if not origin:
					continue
				if versions_by_origin.has_key(origin):
					versions_by_origin[origin].append(version)
				else:
					versions_by_origin[origin] = [version]
		return Status(self.name, self.current_version, self.candidate_version, versions_by_origin)

	def __str__(self):
		"""Returns a string which includes the package name and current and candidate versions."""
		return '<PackageAdapter %s v=%s->%s >' % (self.name, str(self.current_version), str(self.candidate_version))

	__repr__ = __str__



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.reporter import Reporter
from apt_forktracer.status import Status

class TestReporter(test_helper.MoxTestCase):
	def setUp(self):
		super(TestReporter, self).setUp()
		self.reporter = Reporter()

	def testFormattingNoCandidateVersion(self):
		v1 = self.struct()
		v1.string = '1.2'
		status = Status('apackage', v1, None)
		report = self.reporter.format(status)
		self.assertContains(report, '1.2->)')
		self.assertNotContains(report, '[')

	def testFormattingNoCurrentVersion(self):
		v1 = self.struct()
		v1.string = '1.2'
		status = Status('apackage', None, v1)
		report = self.reporter.format(status)
		self.assertContains(report, '(->1.2')
		self.assertNotContains(report, '[')

	def testFormatting(self):
		v1 = self.struct()
		v1.string = '1.2'
		v2 = self.struct()
		v2.string = '1.3'
		v3, v4 = self.struct(), self.struct()
		v3.string = '1.2.3'
		v4.string = 'x.y.z'
		status = Status('apackage', v1, v2, {'Debian': [v3, v4], 'another origin': [v3, v4]})
		report = self.reporter.format(status)
		self.assertContains(report, 'apackage (1.2->1.3) ')
		self.assertContains(report, ' [Debian: 1.2.3 x.y.z]')
		self.assertContains(report, ' [another origin: 1.2.3 x.y.z]')

	def testFormattingSameVersion(self):
		v = self.struct()
		v.string = '1.2'
		status = Status('apackage', v, v)
		report = self.reporter.format(status)
		self.assertNotContains(report, '[')
		self.assertContains(report, 'apackage (1.2)')

	def testReporting(self):
		"""Since this should print to stdout, we don't call it, just check the method's there."""
		mock_status = self.struct()
		self.assert_(self.reporter.report)

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.apt_pkg_adapter import AptPkgAdapter
from apt_forktracer.checker import Checker
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.package_adapter import PackageAdapter
from apt_forktracer.version_adapter import VersionAdapter

class CheckerCheckTestCase(test_helper.MoxTestCase):
	"""Common plumbing."""
	def setUp(self):
		super(CheckerCheckTestCase, self).setUp()
		self.fp = FakePackage()
		self.apt_pkg_adapter = AptPkgAdapter(self._create_mock_apt_pkg_module())
		self.apt_pkg_adapter.init()
		self.setUpChecker()
	def setUpChecker(self):
		self.checker = Checker(self._create_mock_facter('Debian'))
	def _prepare_package_with_candidate_from_official_source_and_current_from_unofficial(self):
		self.fp.append_version(FakeVersion._create('1.2.2', ['NotDebian']), True)
		self.fp.append_version(FakeVersion._create('1.2.3', ['Debian']))
		pa = PackageAdapter(self.fp)
		pa.candidate_version = VersionAdapter(FakeVersion._create('1.2.3', ['Debian']))
		return pa
	def _prepare_package_with_candidate_different_from_current(self):
		self.fp.append_version(FakeVersion._create('1.2.3', ['Debian']), True)
		pa = PackageAdapter(self.fp)
		pa.candidate_version = VersionAdapter((FakeVersion._create('1.2.4', ['Debian'])))
		return pa

class GeneralNonVerboseCheckerCheckTest(CheckerCheckTestCase):
	"""Whether a status is returned depends on the CANDIDATE version, not the
	   currently installed one. This is to cut down on noise when machines are
	   behind WRT upgrades."""
	def test_aborts_on_package_without_versions(self):
		pa = PackageAdapter(self.fp)
		self.assertRaises(ValueError, self.checker.check, pa)
	def test_aborts_on_package_without_current_version(self):
		"""CacheAdapter should not pass such objects to us. It's better to know
		in such cases. If there will ever be a need to support this, it can be
		changed."""
		self.fp.append_version(FakeVersion._create('1.2.3', ['Debian']), True)
		pa = PackageAdapter(self.fp)
		pa.candidate_version = pa.current_version
		pa.current_version = None
		self.assertRaises(ValueError, self.checker.check, pa)
	def test_does_not_abort_and_returns_a_status_object_on_package_without_candidate_version(self):
		"""Because a candidate version is always reported by apt in normal
		situations, even when the only known version is installed and not
		available anywhere else. One case where no candidate version is
		reported is when there is a pin which cannot be met."""
		self.fp.append_version(FakeVersion._create('1.2.3', ['Debian']), True)
		pa = PackageAdapter(self.fp)
		pa.candidate_version = None
		status = self.checker.check(pa)
		self.assert_(status != None)
	def test_returns_none_in_common_case(self):
		"""Common case means: package with identical, officially-available
		current and candidate versions, which is also the newest available
		version."""
		self.fp.append_version(FakeVersion._create('1.2.3', ['Debian']), True)
		self.fp.append_version(FakeVersion._create('1.2.2', ['Debian']))
		pa = PackageAdapter(self.fp)
		pa.candidate_version = pa.current_version
		self.assertEquals(self.checker.check(pa), None)
	def test_returns_none_in_common_case_when_all_versions_available_additionally_from_unofficial_source(self):
		self.fp.append_version(FakeVersion._create('1.2.3', ['Debian', 'UnOfficial']), True)
		self.fp.append_version(FakeVersion._create('1.2.2', ['Debian', 'UnOfficial']))
		pa = PackageAdapter(self.fp)
		pa.candidate_version = pa.current_version
		self.assertEquals(self.checker.check(pa), None)
	def test_returns_a_status_object_on_package_with_same_current_and_candidate_both_only_locally(self):
		self.fp.current_ver = FakeVersion._create('1.2.3', ['dpkg'])
		pa = PackageAdapter(self.fp)
		pa.candidate_version = pa.current_version
		self.assert_(self.checker.check(pa) != None)
	def test_returns_a_status_object_on_package_with_candidate_from_unofficial_source(self):
		self.fp.append_version(FakeVersion._create('1.2.3', ['Unofficial']), True)
		self.fp.append_version(FakeVersion._create('1.2', ['Debian']))
		pa = PackageAdapter(self.fp)
		pa.candidate_version = VersionAdapter(FakeVersion._create('1.2.5', ['NonDebian']))
		status = self.checker.check(pa)
		self.assert_(status != None)
		self.assertEquals(status.package_name, 'afake')
		self.assertEquals(status.installed_version.string, '1.2.3')
		self.assertEquals(status.candidate_version.string, '1.2.5')
	def test_package_without_candidate_version_and_current_unofficial(self):
		self.fp.current_ver = FakeVersion._create('1.2.2', ['NotDebian'])
		pa = PackageAdapter(self.fp)
		pa.candidate_version = None
		status = self.checker.check(pa)
		self.assert_(status != None)

class NonVerboseCheckerCheckTest(GeneralNonVerboseCheckerCheckTest):
	def test_returns_none_on_package_with_candidate_from_official_source_and_current_from_unofficial(self):
		self.assertEquals(self.checker.check(self._prepare_package_with_candidate_from_official_source_and_current_from_unofficial()), None)
	def test_returns_none_when_candidate_different_from_current(self):
		pa = self._prepare_package_with_candidate_different_from_current()
		self.assertEquals(self.checker.check(pa), None)

class VerboseCheckerCheckTest(GeneralNonVerboseCheckerCheckTest):
	"""Returns None in less cases than in the non-verbose mode."""
	def setUpChecker(self):
		self.checker = Checker(self._create_mock_facter('Debian'), True)
	def test_returns_a_status_object_on_package_with_candidate_from_official_source_and_current_from_unofficial(self):
		status = self.checker.check(self._prepare_package_with_candidate_from_official_source_and_current_from_unofficial())
		self.assert_(status != None)
	def test_returns_a_status_object_when_candidate_different_from_current(self):
		pa = self._prepare_package_with_candidate_different_from_current()
		self.assert_(self.checker.check(pa) != None)

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import apt_pkg

from apt_forktracer.testlib import test_helper
from apt_forktracer.apt_pkg_adapter import AptPkgAdapter
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.version_adapter import VersionAdapter

from apt_forktracer.reporter import Reporter

from apt_forktracer.cache_adapter import CacheAdapterFactory

from apt_forktracer.depcache_adapter import DepCacheAdapter
from apt_forktracer.depcache_adapter import DepCacheAdapterFactory

class Base_Apt_Pkg_Adapter_Test(test_helper.MoxTestCase):
	def setUp(self):
		super(Base_Apt_Pkg_Adapter_Test, self).setUp()
		self.create_mock_apt_pkg()
		self.set_up_mock_apt_pkg()
		self.apa = AptPkgAdapter(self.mock_apt_pkg)
		self.set_up_apa()
	def create_mock_apt_pkg(self):
		self.mock_apt_pkg = self._create_mock_apt_pkg_module()
	def set_up_mock_apt_pkg(self):
		pass
	def set_up_apa(self):
		pass

class Newer_Apt_Pkg_Adapter_Test(Base_Apt_Pkg_Adapter_Test):
	def create_mock_apt_pkg(self):
		self.mock_apt_pkg = apt_pkg
	def test_states(self):
		self.mox.ReplayAll()
		self.assertEquals(self.apa.state_installed, apt_pkg.CURSTATE_INSTALLED)
		self.assertEquals(self.apa.state_half_installed, apt_pkg.CURSTATE_HALF_INSTALLED)
		self.assertEquals(self.apa.state_half_configured, apt_pkg.CURSTATE_HALF_CONFIGURED)
		self.assertEquals(self.apa.state_unpacked, apt_pkg.CURSTATE_UNPACKED)

class Uninitialized_Apt_Pkg_Adapter_Test(Base_Apt_Pkg_Adapter_Test):
	def test_methods_fail_on_uninitialized_adapter(self):
		self.mox.ReplayAll()
		self.assertRaisesWithMessageContaining(Exception, 'not initialized', self.apa.get_cache_adapter, 'foo', 'bar', 'baz')
		self.assertRaisesWithMessageContaining(Exception, 'not initialized', self.apa.get_depcache_adapter, 'foo')
		self.assertRaisesWithMessageContaining(Exception, 'not initialized', self.apa.version_compare, 'foo', 'bar')
		self.assertRaisesWithMessageContaining(Exception, 'not initialized', self.apa.version_sort, ['foo', 'bar'])
		self.assertRaisesWithMessageContaining(Exception, 'not initialized', self.apa.version_max, ['foo', 'bar'])

class Initialized_Apt_Pkg_Adapter_Test(Base_Apt_Pkg_Adapter_Test):
	def set_up_mock_apt_pkg(self):
		self.mock_apt_pkg.init_config()
		self.mock_apt_pkg.init_system()
	def set_up_apa(self):
		self.apa.init()
	def test_get_cache_adapters(self):
		mock_reporter = self.mox.CreateMock(Reporter)
		mock_progress = self.struct()

		mock_apt_cache = self.struct()
		mock_cache_adapter = self.struct()
		mock_cache_adapter_factory = self.mox.CreateMock(CacheAdapterFactory)

		mock_apt_depcache = self.struct()
		mock_depcache_adapter = self.mox.CreateMock(DepCacheAdapter)
		mock_depcache_adapter_factory = self.mox.CreateMock(DepCacheAdapterFactory)

		self.mock_apt_pkg.Cache(mock_progress).AndReturn(mock_apt_cache)
		self.mock_apt_pkg.DepCache(mock_apt_cache).AndReturn(mock_apt_depcache)

		mock_cache_adapter_factory.create_cache_adapter(mock_apt_cache, self.apa, mock_reporter).AndReturn(mock_cache_adapter)
		mock_depcache_adapter_factory.create_depcache_adapter(mock_apt_depcache).AndReturn(mock_depcache_adapter)
		self.mox.ReplayAll()

		ca = self.apa.get_cache_adapter(mock_cache_adapter_factory, mock_reporter, mock_progress)
		dca = self.apa.get_depcache_adapter(mock_depcache_adapter_factory)

		self.assertEquals(ca, mock_cache_adapter)
		self.assertEquals(dca, mock_depcache_adapter)
	def test_get_depcache_adapter_fails_without_earlier_get_cache_adapter(self):
		mock_depcache_adapter_factory = self.mox.CreateMock(DepCacheAdapterFactory)
		self.mox.ReplayAll()
		self.assertRaisesWithMessageContaining(Exception, 'must call get_cache_adapter() earlier', self.apa.get_depcache_adapter, mock_depcache_adapter_factory)
	def test_version_comparison(self):
		self.mox.ReplayAll()
		self.assertRaises(ValueError, self.apa.version_compare, None, '1')
		self.assertRaises(ValueError, self.apa.version_compare, '1', None)
		self.assertRaises(ValueError, self.apa.version_compare, None, None)
		self.assert_(self.apa.version_compare('0', '1') < 0)
		self.assert_(self.apa.version_compare('1', '1') == 0)
		self.assert_(self.apa.version_compare('1', '0') > 0)
	def test_version_sort(self):
		self.mox.ReplayAll()
		self.assertEquals(self.apa.version_sort([]), [])
		v1 = VersionAdapter(FakeVersion._create('1', []))
		v0 = VersionAdapter(FakeVersion._create('0', []))
		v2 = VersionAdapter(FakeVersion._create('2.0', []))
		v21 = VersionAdapter(FakeVersion._create('2.0~1', []))
		self.assertEquals(self.apa.version_sort([v21]), [v21])
		self.assertEquals(self.apa.version_sort([v1, v0, v21, v2]), [v2, v21, v1, v0])
	def test_version_max_returns_None_on_empty_list(self):
		self.mox.ReplayAll()
		self.assertEquals(self.apa.version_max([]), None)
		v2 = VersionAdapter(FakeVersion._create('2.0', []))
		v21 = VersionAdapter(FakeVersion._create('2.0~1', []))
		self.assertEquals(self.apa.version_max([v21]), v21)
		self.assertEquals(self.apa.version_max([v2, v21]), v2)
		self.assertEquals(self.apa.version_max([v21, v2]), v2)

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os

class _ConfigFinderIterator:
	"""Iterator class used by ConfigFinder. See that class for more information."""

	def __init__(self, cf):
		self.cf = cf
		self.index = 0
		self.file = None

	def __iter__(self):
		return self

	def next(self):
		if len(self.cf.paths) > self.index:
			path = self.cf.paths[self.index]
			if self.file:
				self.file.close()
			self.file = open(path)
			self.index += 1
			return (path, self.file)
		else:
			if self.file:
				self.file.close()
			raise StopIteration()

class ConfigFinder:
	"""Provides a method to get all config files. Arguments are files or
	directories to look for. Files are opened, and directories are searched
	looking for files which end with '.conf' but do not start with a dot.

	Iterating over this object returns tuples (path, file), where path is the
	path to a config file, and file is the open file object.
	"""

	def __init__(self, *paths):
		self.paths = []
		for path in paths:
			if not os.path.exists(path):
				continue
			if os.path.isdir(path):
				for name in os.listdir(path):
					if name and name[0] != '.' and name.endswith('.conf'):
						self.paths.append(os.path.join(path, name))
			else:
				self.paths.append(path)

	def __iter__(self):
		return _ConfigFinderIterator(self)




#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.config_finder import ConfigFinder

class Test_Config_Finder(test_helper.MoxTestCase):
	def test(self):
		cf = ConfigFinder('test-data/config', 'test-data/not_exists', 'test-data/config.d')
		entries = 0
		plan = [
			('test-data/config', 4),
			('test-data/config.d/sub.conf', 0),
			('INVALID ENTRY to catch unexpected files', 0)
		]
		for path, file in cf:
			self.assertEquals(path, plan[entries][0])
			self.assert_(file.readlines() > plan[entries][1])
			entries += 1
		self.assertEquals(entries, 2)


if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import apt_pkg
import mox
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.apt_pkg_adapter import AptPkgAdapter
from apt_forktracer.cache_adapter import CacheAdapterFactory
from apt_forktracer.checker import Checker
from apt_forktracer.config import Config
from apt_forktracer.config_finder import ConfigFinder
from apt_forktracer.config_parser import ConfigParser
from apt_forktracer.depcache_adapter import DepCacheAdapter
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.package_adapter import PackageAdapterFactory
from apt_forktracer.policy import Policy, VerbosePolicy
from apt_forktracer.reporter import Reporter
from apt_forktracer.facter import Facter

class TestIntegraton(test_helper.MoxTestCase):
	def setUp(self):
		super(TestIntegraton, self).setUp()
		dpkg_status_file = FakePackageFile(type = 'dpkg')
		debian_stable_package_file = FakePackageFile(archive = 'stable')
		debian_proposed_updates_package_file = FakePackageFile(archive = 'stable-proposed-updates')
		debian_security_package_file = FakePackageFile(archive = 'stable-security')
		local_package_file = FakePackageFile(origin = 'SnakeOil, Inc.', archive = 'etch')

		libc6_version = FakeVersion('2.6.1-2etch1')
		libc6_version.append_package_file(debian_stable_package_file)
		libc6_version.append_package_file(debian_security_package_file)
		libc6_updates_version = FakeVersion('2.6.1-2etch2')
		libc6_updates_version.append_package_file(debian_proposed_updates_package_file)
		libc6_updates_version.append_package_file(dpkg_status_file)
		libc6 = FakePackage(name = 'libc6')
		libc6.append_version(libc6_updates_version, True)
		libc6.append_version(libc6_version)

		libspf_version = FakeVersion('0.1-1')
		libspf_version.append_package_file(debian_stable_package_file)
		libspf_updates_version = FakeVersion('0.1-2')
		libspf_updates_version.append_package_file(debian_proposed_updates_package_file)
		libspf_local_version = FakeVersion('0.1-1~sl1')
		libspf_local_version.append_package_file(dpkg_status_file)
		libspf_local_version.append_package_file(local_package_file)
		libspf = FakePackage(name = 'libspf')
		libspf.append_version(libspf_version)
		libspf.append_version(libspf_local_version, True)
		libspf.append_version(libspf_updates_version)

		libfoobar_version = FakeVersion('0.5-5')
		libfoobar_version.append_package_file(debian_stable_package_file)
		libfoobar_local_version = FakeVersion('0.5-5~sl1')
		libfoobar_local_version.append_package_file(dpkg_status_file)
		libfoobar_local_version.append_package_file(local_package_file)
		libfoobar = FakePackage(name = 'libfoobar')
		libfoobar.append_version(libfoobar_version)
		libfoobar.append_version(libfoobar_local_version, True)

		git_version = FakeVersion('1:1.5.2.5-2build1')
		git_version.append_package_file(debian_stable_package_file)
		git_backport_version = FakeVersion('1:1.5.6.3-1.1ubuntu2~mowsiany.1')
		git_backport_version.append_package_file(dpkg_status_file)
		git_backport_version.append_package_file(local_package_file)
		git = FakePackage(name = 'git-core')
		git.append_version(git_version)
		git.append_version(git_backport_version, True)

		self.apt_cache = self.struct()
		self.apt_cache.packages = [git, libc6, libspf, libfoobar]

		self.apt_depcache = self.struct()
		version_table = {
			'libc6': libc6_updates_version,
			'libspf': libspf_local_version,
			'libfoobar': libfoobar_local_version,
			'git-core': git_backport_version}
		self.apt_depcache.get_candidate_ver = lambda o: version_table[o.name]

		self.reporter = self.mox.CreateMock(Reporter)
		self.mock_progress = self.struct()

		self.apt_pkg = self._create_mock_apt_pkg_module()
		test_helper.copy_state_constants(self.apt_pkg, apt_pkg)
		self.apt_pkg.Cache(self.mock_progress).AndReturn(self.apt_cache)

		self.facter = self.mox.CreateMock(Facter)
		self.facter.distributors_id = 'Debian'

	def finishSetUp(self):
		self.apt_pkg_adapter = AptPkgAdapter(self.apt_pkg)
		self.apt_pkg_adapter.init()
		cache_adapter_factory = CacheAdapterFactory()
		self.package_adapter_factory = PackageAdapterFactory(DepCacheAdapter(self.apt_depcache))
		self.cache_adapter = self.apt_pkg_adapter.get_cache_adapter(cache_adapter_factory, self.reporter, self.mock_progress)

		config_finder = ConfigFinder('test-data/config')
		config_parser = ConfigParser()
		self.config = Config()
		for path, file in config_finder:
			config_parser.load(file, self.config)

	def test_verbose(self):
		# lib6 - never
		self.reporter.report(mox.Func(lambda o: o.package_name == 'libspf')).InAnyOrder()
		self.reporter.report(mox.Func(lambda o: o.package_name == 'libfoobar')).InAnyOrder()
		self.reporter.report(mox.Func(lambda o: o.package_name == 'git-core')).InAnyOrder()
		self.mox.ReplayAll()

		self.finishSetUp()
		checker = Checker(self.facter, True)
		policy = VerbosePolicy()
		self.cache_adapter.run(checker, policy, self.package_adapter_factory)

	def test_non_verbose_empty_config(self):
		# libc6 - never
		self.reporter.report(mox.Func(lambda o: o.package_name == 'libspf')).InAnyOrder()
		# libfoobar - never
		self.reporter.report(mox.Func(lambda o: o.package_name == 'git-core')).InAnyOrder()
		self.mox.ReplayAll()

		self.finishSetUp()
		checker = Checker(self.facter)
		policy = Policy(self.apt_pkg_adapter, self.facter, Config())
		self.cache_adapter.run(checker, policy, self.package_adapter_factory)

	def test_non_verbose(self):
		# libc6 - never
		self.reporter.report(mox.Func(lambda o: o.package_name == 'libspf'))
		# libfoobar - never
		# git-core - never
		self.mox.ReplayAll()

		self.finishSetUp()
		checker = Checker(self.facter)
		policy = Policy(self.apt_pkg_adapter, self.facter, self.config)
		self.cache_adapter.run(checker, policy, self.package_adapter_factory)

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.config import Config
from apt_forktracer.config_stanza import ConfigStanza

class Test_Config(test_helper.MoxTestCase):
	def setUp(self):
		super(Test_Config, self).setUp()
		self.c = Config()
	def test_addition_and_retrieval(self):
		foo_stanza1 = self.mox.CreateMock(ConfigStanza)
		foo_stanza1.get('package').AndReturn('foo')
		foo_stanza2 = self.mox.CreateMock(ConfigStanza)
		foo_stanza2.get('package').AndReturn('foo')
		bar_stanza = self.mox.CreateMock(ConfigStanza)
		bar_stanza.get('package').AndReturn('bar')
		self.mox.ReplayAll()

		self.c.add(foo_stanza1)
		self.c.add(foo_stanza2)
		self.c.add(bar_stanza)
		foo_stanzas = self.c.package('foo')
		bar_stanzas = self.c.package('bar')
		baz_stanzas = self.c.package('baz')
		self.assertEquals(len(foo_stanzas), 2)
		self.assertEquals(foo_stanzas[0], foo_stanza1)
		self.assertEquals(foo_stanzas[1], foo_stanza2)
		self.assertEquals(len(bar_stanzas), 1)
		self.assertEquals(bar_stanzas[0], bar_stanza)
		self.assertEquals(len(baz_stanzas), 0)


if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.apt_pkg_adapter import AptPkgAdapter
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.package_adapter import PackageAdapter

class TestBasePackageAdapter(test_helper.MoxTestCase):
	def setUp(self):
		super(TestBasePackageAdapter, self).setUp()
		self.fake_package = FakePackage()
		self.setUp_mangle_fake_package()
		self.pa = PackageAdapter(self.fake_package)
		self.setUp_mangle_package_adapter()
		self.facter = self._create_mock_facter('Debian')
		self.apt_pkg_adapter = AptPkgAdapter(self._create_mock_apt_pkg_module())
		self.apt_pkg_adapter.init()
		self.status = self.pa.get_status(self.facter)
	def setUp_mangle_fake_package(self):
		pass
	def setUp_mangle_package_adapter(self):
		pass
	def testProperties(self):
		self.assertEquals(self.pa.name, 'afake')
		self.assertEquals(self.pa.apt_package, self.fake_package)
	def testLenOfVersionsNonNegative(self):
		self.assert_(self.pa.versions >= 0)
	def testBasicStringificationWorks(self):
		self.assertContains(str(self.pa), 'PackageAdapter')
	def setUpAddAVersion(self, source = 'NonDebian', version = 'blah'):
		self.fake_package.append_version(FakeVersion._create(version, [source]))
	def test_get_status_returns_an_object(self):
		self.assert_(self.status)
		self.assertEquals(self.status.package_name, 'afake')
		self.assertEquals(self.status.installed_version, self.pa.current_version)
		self.assertEquals(self.status.candidate_version, self.pa.candidate_version)
		self.assert_(len(self.status.versions_by_origin) >= 0)

class TestZeroVersionPackageAdapter(TestBasePackageAdapter):
	def testEmptyVersions(self):
		self.assertEquals(self.pa.versions, [])
		self.assertEquals(len(self.status.versions_by_origin), 0)
	def testStringificationWorks(self):
		self.assertContains(str(self.pa), 'v=None->None')

class TestOneOfficialVersionPackageAdapter(TestBasePackageAdapter):
	def setUp_mangle_fake_package(self):
		self.setUpAddAVersion('Debian')
	def testOneVersion(self):
		self.assertEquals(len(self.pa.versions), 1)
		self.assertEquals(len(self.status.versions_by_origin), 1)

class TestOneUnofficialVersionPackageAdapter(TestBasePackageAdapter):
	def setUp_mangle_fake_package(self):
		self.setUpAddAVersion()
	def testOneVersion(self):
		self.assertEquals(len(self.pa.versions), 1)
		self.assertEquals(len(self.status.versions_by_origin), 1)

class TestFourVersionPackageAdapter(TestBasePackageAdapter):
	def setUp_mangle_fake_package(self):
		self.setUpAddAVersion('NonDebian', '0')
		self.setUpAddAVersion('NonDebian', '4')
		self.setUpAddAVersion('Debian', '1')
		self.setUpAddAVersion('Debian', '2')
		self.setUpAddAVersion('dpkg', '5')
	def testFourVersions(self):
		self.assertEquals(len(self.pa.versions), 5)
		self.assertEquals(len(self.status.versions_by_origin), 2)
		self.assertEquals(len(self.status.versions_by_origin['Debian']), 2)
		self.assertEquals(len(self.status.versions_by_origin['NonDebian']), 2)

class TestPackageAdapterWithInstalledVersion(TestBasePackageAdapter):
	def setUp_mangle_fake_package(self):
		self.setUpAddAVersion()
		self.fake_package.current_ver = FakeVersion('1.xx.yy')
		self.official_version = FakeVersion._create('2.3.4', ['Debian'])
		self.fake_package.append_version(self.official_version)
	def setUp_mangle_package_adapter(self):
		self.pa.candidate_version = '2.3.4'
	def test_versions(self):
		self.assertEquals(self.pa.current_version.string, '1.xx.yy')
		self.assertEquals(len(self.status.versions_by_origin), 2)
	def testStringificationWorks(self):
		self.assertMatches(str(self.pa), r'VersionAdapter.*1\.xx\.yy.*->2\.3\.4')

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.package_file_adapter import PackageFileAdapter

class BasePFATest(test_helper.MoxTestCase):
	def setUpPF(self):
		self.fake_package_file = FakePackageFile()
	def setUp(self):
		super(BasePFATest, self).setUp()
		self.setUpPF()
		self.pfa = PackageFileAdapter(self.fake_package_file)
		self.mock_debian_facter = self._create_mock_facter('Debian')
		self.mock_ubuntu_facter = self._create_mock_facter('Ubuntu')

class BasePFATestWithOfficial(BasePFATest):
	def testIsOfficialTrue(self):
		self.assert_(self.pfa.is_official(self.mock_debian_facter))
	def testIsOfficialFalse(self):
		self.assert_(not self.pfa.is_official(self.mock_ubuntu_facter))

class TestBasePackageFileAdapter(BasePFATestWithOfficial):
	def testAttributes(self):
		self.assertEquals(self.pfa.name, '/a/fake')
		self.assertEquals(self.pfa.archive, 'stable-proposed-updates')
		self.assertEquals(self.pfa.component, 'main')
		self.assertEquals(self.pfa.version, '1.0')
		self.assertEquals(self.pfa.origin, 'Debian')
		self.assertEquals(self.pfa.label, 'Debian')
		self.assertEquals(self.pfa.not_automatic, 0)
		self.assertEquals(self.pfa.index_type, 'Debian Package Index')
	def testStringification(self):
		self.assertMatches(str(self.pfa), '<PackageFileAdapter path=/a/fake a=stable-proposed-updates c=main v=1.0 o=Debian l=Debian>')

class TestBasePackageFileAdapterNonAuto(BasePFATestWithOfficial):
	def setUpPF(self):
		BasePFATest.setUpPF(self)
		self.fake_package_file.not_automatic = 1
	def testAttributes(self):
		self.assertEquals(self.pfa.name, '/a/fake')
		self.assertEquals(self.pfa.archive, 'stable-proposed-updates')
		self.assertEquals(self.pfa.component, 'main')
		self.assertEquals(self.pfa.version, '1.0')
		self.assertEquals(self.pfa.origin, 'Debian')
		self.assertEquals(self.pfa.label, 'Debian')
		self.assertEquals(self.pfa.not_automatic, 1)
		self.assertEquals(self.pfa.index_type, 'Debian Package Index')
	def testStringification(self):
		self.assertMatches(str(self.pfa), '<PackageFileAdapter path=/a/fake a=stable-proposed-updates c=main v=1.0 o=Debian l=Debian NONAUTO>')

class TestBasePackageFileAdapterDpkgStatus(BasePFATest):
	def setUpPF(self):
		self.fake_package_file = FakePackageFile(type = 'dpkg')
	def testAttributes(self):
		self.assertEquals(self.pfa.name, '/var/lib/dpkg/status')
		self.assertEquals(self.pfa.index_type, 'Debian dpkg status file')
	def testStringification(self):
		self.assertMatches(str(self.pfa), '<PackageFileAdapter\(dpkg status\) path=/var/lib/dpkg/status>')
	def testIsOfficialFalse(self):
		self.assert_(not self.pfa.is_official(self.mock_debian_facter))
		self.assert_(not self.pfa.is_official(self.mock_ubuntu_facter))

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.apt_pkg_adapter import AptPkgAdapter
from apt_forktracer.config_stanza import ConfigStanza
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.package_file_adapter import PackageFileAdapter
from apt_forktracer.version_adapter import VersionAdapter
from apt_forktracer.policy import Policy
from apt_forktracer.status import Status

	
class TestPolicyBase(test_helper.MoxTestCase):
	def _create_mock_version_adapter(self, version_string):
		if version_string == None:
			return None
		mva = self.mox.CreateMock(VersionAdapter)
		mva.files = []
		if type(version_string) == tuple:
			mva.string = version_string[0]
			for o in version_string[1]:
				mva.files.append(PackageFileAdapter(FakePackageFile(origin = o)))
		else:
			mva.string = version_string
			mva.files.append(PackageFileAdapter(FakePackageFile(origin = 'Debian')))
		return mva
	def _create_mock_status(self, package, current_version, candidate_version, _official_versions, other_versions = {}):
		official_versions = [self._create_mock_version_adapter(v) for v in _official_versions]
		vbo = {}
		vbo['Debian'] = official_versions
		for o, vers in other_versions.items():
			vbo[o] = [self._create_mock_version_adapter((v, o,)) for v in vers]
		return Status(package, self._create_mock_version_adapter(current_version), self._create_mock_version_adapter(candidate_version), vbo)
	def _create_mock_config(self, hashes):
		stanzas = []
		for hash in hashes:
			stanza = ConfigStanza()
			stanza.set('package', 'apackage', 1)
			stanza.set('accept-origin', hash['accepted_origin'], 2)
			stanza.set('track-origin', hash['track_origin'], 3)
			stanza.set('track-version', hash['track_version'], 4)
			stanza.finish(4)
			stanzas.append(stanza)
		mock_config = self.struct()
		mock_config.package = lambda name: name == 'apackage' and stanzas or []
		return mock_config
	def setUp(self):
		super(TestPolicyBase, self).setUp()
		self.apt_pkg_adapter = AptPkgAdapter(self._create_mock_apt_pkg_module())
		self.apt_pkg_adapter.init()
		self.mock_facter = self._create_mock_facter('Debian')
		self.set_up_policy_creation()
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter, self._create_mock_config([]))
	def assert_should_report_yes(self, current_version, candidate_version, official_versions, package = 'apackage', other_versions = {}):
		s = self._create_mock_status(package, current_version, candidate_version, official_versions, other_versions = other_versions)
		self.assert_(self.policy.should_report(s))
	def assert_should_report_NOT(self, current_version, candidate_version, official_versions, package = 'apackage', other_versions = {}):
		s = self._create_mock_status(package, current_version, candidate_version, official_versions, other_versions = other_versions)
		self.assert_(not self.policy.should_report(s))
	def test_missing_version(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', None, ['2.1'])
		self.assert_should_report_yes(None, '2.1~1', ['2.1'])
		self.assert_should_report_yes('2.1~1', '2.1~1', [])

class Test_Policy_Base_Version(TestPolicyBase):
	def test_base_version(self):
		self.mox.ReplayAll()
		self.assertRaises(TypeError, self.policy.base, None)
		self.assertEquals(self.policy.base(''), '')
		self.assertEquals(self.policy.base('1'), '1')
		self.assertEquals(self.policy.base('1~1'), '1')
		self.assertEquals(self.policy.base('1~1~2'), '1~1')

class Test_Policy_Should_Report_With_Same_Candidate_Version_As_Installed(TestPolicyBase):
	def test_official_not_available(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.1~1', [])
	def test_candidate_newer_than_available_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.1~1', ['2.0'])
	def test_candidate_same_as_latest_official(self):
		self.mox.ReplayAll()
		# This would probably be skipped by Checker, but in case it wasn't, we should not report it.
		self.assert_should_report_NOT('2.1', '2.1', ['2.1'])
		self.assert_should_report_NOT('2.1~1', '2.1~1', ['2.1~1'])
	def test_candidate_derived_directly_from_latest_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT('2.1~1', '2.1~1', ['2.1'])
	def test_candidate_derived_from_previous_official_version(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.1~1', ['2.2', '2.1'])

class Test_Policy_Should_Report_With_Newer_Candidate_Version_Than_Installed(TestPolicyBase):
	"""A candidate version newer than the installed one means that an upgrade
	   is pending. If the _candidate_ version is recent enough not to cause a
	   report, then reporting the status based on the _currently_ installed
	   version will cause noise on machines which have not yet been upgraded.
	   Therefore we report what the situation would be _after_ the upgrade, as
	   the system administrator has other tools to notify her when there are
	   pending upgrades. The VerbosePolicy can be used to display all packages
	   identified by Checker."""
	# none available
	def test_official_not_available(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.1~2', [])
	# newer than available
	def test_installed_and_candidate_package_newer_than_available_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.1~2', ['2.0'])
	def test_installed_package_derived_directly_from_latest_official_while_candidate_is_even_newer(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.2~2', ['2.1'])
	# same as or derived from newest available
	def test_installed_and_candidate_package_derived_directly_from_latest_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT('2.1~1', '2.1~2', ['2.1'])
		self.assert_should_report_NOT('2.1~1', '2.1', ['2.1'])
		self.assert_should_report_NOT('2.1~1', '2.1~2', ['2.1~2'])
	def test_installed_package_derived_from_previous_official_version_while_candidate_derived_from_newest_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT('2.1~1', '2.2~1', ['2.2', '2.1'])
		self.assert_should_report_NOT('2.1~1', '2.2', ['2.2', '2.1'])
		self.assert_should_report_NOT('2.1~1', '2.2~2', ['2.2~2', '2.1'])
	# older than available
	def test_both_installed_and_candidate_package_derived_from_previous_official_version(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.1~2', ['2.2', '2.1'])

class Test_Policy_Should_Report_With_Older_Candidate_Version_Than_Installed(TestPolicyBase):
	"""An older candidate version means that a downgrade is pending. This calls
	   for the same behaviour as described in
	   Test_Policy_Should_Report_With_Newer_Candidate_Version_Than_Installed."""
	# none available
	def test_official_not_available(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~2', '2.1~1', [])
	# newer than available
	def test_both_installed_and_candidate_package_newer_than_available_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~2', '2.1~1', ['2.0'])
	# same as or derived from available
	def test_both_installed_and_candidate_package_newer_than_available_official(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT('2.1~2', '2.0~1', ['2.0'])
		self.assert_should_report_NOT('2.1~2', '2.0', ['2.0'])
		self.assert_should_report_NOT('2.1~2', '2.0~1', ['2.0~1'])
	# older than available
	def test_installed_package_derived_directly_from_latest_official_while_candidate(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes('2.1~1', '2.0~1', ['2.1'])


class Test_Policy_With_Config(TestPolicyBase):
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter,
			self._create_mock_config([
				{'accepted_origin': 'accepted origin', 'track_origin': 'foo', 'track_version': '1.0'},
				{'accepted_origin': 'accepted origin', 'track_origin': 'Debian', 'track_version': '2.0'}]))
	def test_configured_origin_is_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['2.0'])
		self.assert_should_report_NOT(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['1.0'], other_versions = {'foo': ['1.0']})
	def test_unconfigured_origin_is_not_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2',   ['ANOTHER origin']),  ('2.2',   ['ANOTHER origin']),  ['2.0'])
	def test_configured_origin_is_not_ignored_if_official_version_does_not_meet_condition(self):
		self.mox.ReplayAll()
		# official sources have too new a version
		self.assert_should_report_yes(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['2.1'])
		# official sources have too old version
		self.assert_should_report_yes(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['1.9'])
		# official sources have no versions at all
		self.assert_should_report_yes(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), [])
	def test_default_rule_not_referenced_if_config_is_provided_for_a_given_package(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.0~1', ['ANOTHER origin']),  ('2.0~1', ['ANOTHER origin']),  ['2.0'])
	def test_default_rules_are_referenced_for_packages_without_configuration(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2',   ['ANOTHER origin']),  ('2.2',   ['ANOTHER origin']),  ['2.0'], package = 'package_without_config')
		self.assert_should_report_NOT(('2.0~1', ['ANOTHER origin']),  ('2.0~1', ['ANOTHER origin']),  ['2.0'], package = 'package_without_config')

class Test_Policy_With_Config_Track_Candidate_Version(TestPolicyBase):
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter, self._create_mock_config([{'accepted_origin': 'accepted origin', 'track_origin': 'Debian', 'track_version': '=candidate'}]))
	def test_configured_origin_is_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['2.2'])
	def test_unconfigured_origin_is_not_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2',   ['ANOTHER origin']),  ('2.2',   ['ANOTHER origin']),  ['2.2'])
	def test_configured_origin_is_not_ignored_if_official_version_does_not_meet_condition(self):
		self.mox.ReplayAll()
		# official sources have too new a version
		self.assert_should_report_yes(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['2.3'])
		# official sources have too old version
		self.assert_should_report_yes(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), ['2.1'])
		# official sources have no versions at all
		self.assert_should_report_yes(('2.2',   ['accepted origin']), ('2.2',   ['accepted origin']), [])
	def test_default_rule_not_referenced_if_config_is_provided_for_a_given_package(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.0~1', ['ANOTHER origin']),  ('2.0~1', ['ANOTHER origin']),  ['2.0'])
	def test_default_rules_are_referenced_for_packages_without_configuration(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2',   ['ANOTHER origin']),  ('2.2',   ['ANOTHER origin']),  ['2.0'], package = 'package_without_config')
		self.assert_should_report_NOT(('2.0~1', ['ANOTHER origin']),  ('2.0~1', ['ANOTHER origin']),  ['2.0'], package = 'package_without_config')

class Test_Policy_With_Config_Track_Candidate_Base_Version(TestPolicyBase):
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter, self._create_mock_config([{'accepted_origin': 'accepted origin', 'track_origin': 'Debian', 'track_version': '=candidate-base'}]))
	def test_configured_origin_is_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.2~1',   ['accepted origin']), ('2.2~1',   ['accepted origin']), ['2.2'])
	def test_unconfigured_origin_is_not_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2~1',   ['ANOTHER origin']),  ('2.2~1',   ['ANOTHER origin']),  ['2.2'])
	def test_configured_origin_is_not_ignored_if_official_version_does_not_meet_condition(self):
		self.mox.ReplayAll()
		# official sources have too new a version
		self.assert_should_report_yes(('2.2~1',   ['accepted origin']), ('2.2~1',   ['accepted origin']), ['2.3'])
		# official sources have too old version
		self.assert_should_report_yes(('2.2~1',   ['accepted origin']), ('2.2~1',   ['accepted origin']), ['2.1'])
		# official sources have no versions at all
		self.assert_should_report_yes(('2.2~1',   ['accepted origin']), ('2.2~1',   ['accepted origin']), [])
	def test_default_rule_not_referenced_if_config_is_provided_for_a_given_package(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.0~1', ['ANOTHER origin']),  ('2.0~1', ['ANOTHER origin']),  ['2.0'])
	def test_default_rules_are_referenced_for_packages_without_configuration(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2',   ['ANOTHER origin']),  ('2.2',   ['ANOTHER origin']),  ['2.0'], package = 'package_without_config')
		self.assert_should_report_NOT(('2.0~1', ['ANOTHER origin']),  ('2.0~1', ['ANOTHER origin']),  ['2.0'], package = 'package_without_config')

class Test_Policy_With_Config_For_Non_Debian_Tracked_Origin(TestPolicyBase):
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter, self._create_mock_config([{'accepted_origin': 'accepted origin', 'track_origin': 'tracked origin', 'track_version': '2.0'}]))
	def test_configured_origin_is_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.2', ['accepted origin']),  ('2.2',   ['accepted origin']), ['2.0'], other_versions = {'tracked origin': ['2.0']})
	def test_only_configured_origin_is_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2', ['ANOTHER origin']),   ('2.2',   ['ANOTHER origin']),  ['2.0'], other_versions = {'tracked origin': ['2.0']})
	def test_origin_is_not_ignored_if_tracked_version_does_not_meet_condition(self):
		self.mox.ReplayAll()
		# tracked origin has too new a version
		self.assert_should_report_yes(('2.2', ['accepted origin']),  ('2.2',   ['accepted origin']), ['2.0'], other_versions = {'tracked origin': ['2.1']})
		# tracked origin has too old version
		self.assert_should_report_yes(('2.2', ['accepted origin']),  ('2.2',   ['accepted origin']), ['2.0'], other_versions = {'tracked origin': ['1.9']})
		# tracked origin has no version at all
		self.assert_should_report_yes(('2.2', ['accepted origin']),  ('2.2',   ['accepted origin']), ['2.0'], other_versions = {'some other origin': ['2.0']})
		self.assert_should_report_yes(('2.2', ['accepted origin']),  ('2.2',   ['accepted origin']), ['2.0'], other_versions = {})
	def test_default_rule_not_referenced_if_config_is_provided_for_a_given_package(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.0~1', ['ANOTHER origin']), ('2.0~1', ['ANOTHER origin']),  ['2.0'], other_versions = {'tracked origin': ['2.0']})
	def test_default_rule_IS_referenced_for_packages_without_configuration(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.0~1', ['ANOTHER origin']), ('2.0~1', ['ANOTHER origin']),  ['2.0'], package = 'package_without_config', other_versions = {'tracked origin': ['2.0']})

class Test_Policy_With_Config_For_Any_Accepted_Origin(TestPolicyBase):
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter, self._create_mock_config([{'accepted_origin': '*', 'track_origin': 'tracked origin', 'track_version': '2.0'}]))
	def test_any_origin_is_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.2',  ['accepted origin']), ('2.2', ['whatever origin']),  ['2.0'], other_versions = {'tracked origin': ['2.0']})
	def test_origin_is_not_ignored_if_tracked_version_does_not_meet_condition(self):
		self.mox.ReplayAll()
		# tracked origin has too new a version
		self.assert_should_report_yes(('2.2',  ['accepted origin']), ('2.2', ['whatever origin']),  ['2.0'], other_versions = {'tracked origin': ['2.1']})
		# tracked origin has too old version
		self.assert_should_report_yes(('2.2',  ['accepted origin']), ('2.2', ['whatever origin']),  ['2.0'], other_versions = {'tracked origin': ['1.9']})
		# tracked origin has no version at all
		self.assert_should_report_yes(('2.2',  ['accepted origin']), ('2.2', ['whatever origin']),  ['2.0'], other_versions = {'some other origin': ['2.0']})
		self.assert_should_report_yes(('2.2',  ['accepted origin']), ('2.2', ['whatever origin']),  ['2.0'], other_versions = {})
	def test_default_rule_IS_referenced_for_packages_without_configuration(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.0~1', ['ANOTHER origin']), ('2.0~1', ['ANOTHER origin']), ['2.0'], package = 'package_without_config', other_versions = {'tracked origin': ['2.0']})

class Test_Policy_With_Config_For_Any_Tracked_Origin(TestPolicyBase):
	def set_up_policy_creation(self):
		self.policy = Policy(self.apt_pkg_adapter, self.mock_facter, self._create_mock_config([{'accepted_origin': 'accepted origin', 'track_origin': '*', 'track_version': '2.0'}]))
	def test_origin_is_ignored_if_ANY_origin_meets_condition(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.2', ['accepted origin']), ('2.2', ['accepted origin']), ['2.0'], other_versions = {'whatever origin': ['2.0']})
	def test_origin_is_not_ignored_if_NO_origin_meets_version_condition(self):
		self.mox.ReplayAll()
		# too new version
		self.assert_should_report_yes(('2.2', ['accepted origin']), ('2.2', ['accepted origin']), ['2.0'], other_versions = {'whatever origin': ['2.1']})
		# too old version
		self.assert_should_report_yes(('2.2', ['accepted origin']), ('2.2', ['accepted origin']), ['2.1'], other_versions = {'whatever origin': ['1.9']})
		# no versions at all
		self.assert_should_report_yes(('2.2', ['accepted origin']), ('2.2', ['accepted origin']), ['2.1'], other_versions = {})
	def test_unconfigured_origin_is_not_ignored(self):
		self.mox.ReplayAll()
		self.assert_should_report_yes(('2.2', ['accepted origin']), ('2.2', ['ANOTHER origin']),  ['2.0'], other_versions = {'whatever origin': ['2.0']})
	def test_default_rule_IS_referenced_for_packages_without_configuration(self):
		self.mox.ReplayAll()
		self.assert_should_report_NOT(('2.0~1', ['ANOTHER origin']), ('2.0~1', ['ANOTHER origin']), ['2.0'], package = 'package_without_config', other_versions = {'tracked origin': ['2.0']})

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class CacheAdapterFactory:
	"""A factory of CacheAdapter objects."""
	def create_cache_adapter(self, apt_cache, apt_pkg_adapter, reporter):
		return _CacheAdapter(apt_cache, apt_pkg_adapter, reporter)

class _CacheAdapter:
	"""Wrapper for the pkgCache class. See AptPkgAdapter for why this class is necessary."""
	def __init__(self, apt_cache, apt_pkg_adapter, reporter):
		self.apt_cache = apt_cache
		self.reporter = reporter
		self.states_we_check = [ apt_pkg_adapter.state_installed, apt_pkg_adapter.state_half_configured, apt_pkg_adapter.state_half_installed, apt_pkg_adapter.state_unpacked ]
	def run(self, checker, policy, package_adapter_factory):
		for package in self.apt_cache.packages:
			if package.current_state not in self.states_we_check:
				continue
			pa = package_adapter_factory.create_package_adapter(package)
			status = checker.check(pa)
			if not status:
				continue
			if policy.should_report(status):
				self.reporter.report(status)

	def __str__(self):
		return '<CacheAdapter with %d package(s)>' % len(self.apt_cache.packages)
	__repr__ = __str__




#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import apt_pkg
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.depcache_adapter import DepCacheAdapterFactory
from apt_forktracer.package_adapter import PackageAdapter
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.testlib.fake_version import FakeVersion

class TestDepCacheAdapter(test_helper.MoxTestCase):
	def setUp(self):
		super(TestDepCacheAdapter, self).setUp()
		self.mock_depcache = self.mox.CreateMock(apt_pkg.DepCache)
		self.a_fake_package = FakePackage()
		self.package_adapter = PackageAdapter(self.a_fake_package)
		self.dca = DepCacheAdapterFactory().create_depcache_adapter(self.mock_depcache)
	def testNoCandidate(self):
		self.mock_depcache.get_candidate_ver(self.a_fake_package).AndReturn(None)
		self.mox.ReplayAll()

		version_adapter = self.dca.get_candidate_version(self.package_adapter)
		self.assertEquals(version_adapter, None)
	def testWithCandidate(self):
		fake_version = FakeVersion('1.2')
		fake_version.append_package_file(FakePackageFile())
		self.mock_depcache.get_candidate_ver(self.a_fake_package).AndReturn(fake_version)
		self.mox.ReplayAll()

		version_adapter = self.dca.get_candidate_version(self.package_adapter)
		self.assertEquals(version_adapter.string, '1.2')

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class VersionChecker:
	"""Given a VersionAdapter object, determines whether it is sufficiently
	non-standard (i.e. non-official, or inconsistent state) to be reported.
	Checks whether the version is available and official. Does not do detailed
	version string analyses (this is the job of Policy, which also decides
	whether to ignore any reports). See also Checker, which invokes methods of
	this class."""
	def __init__(self, facter):
		self.facter = facter

	def analyze(self, version):
		"""Returns True if this version is somehow suspicious.
		Currently just checks whether a version is officially available."""
		return not version.is_officially_available(self.facter)



__all__ = [ 'test_adapters', 'test_apt_pkg_adapter', 'test_cache_adapter', 'test_checker', 'test_config', 'test_config_finder', 'test_config_parser', 'test_config_stanza', 'test_depcache_adapter', 'test_facter', 'test_integration', 'test_package_adapter', 'test_package_file_adapter', 'test_policy', 'test_reporter', 'test_status', 'test_verbose_policy', 'test_version_adapter', 'test_version_checker', ]



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from apt_forktracer.package_file_adapter import PackageFileAdapter

class FakePackageFile:
	"""Pretends to be the pkgCache::PackageFile object from apt_pkg. We cannot
	use the real one, because it is tied to the binary cache, which is
	difficult to construct and control."""
	def __init__(self, path = '/a/fake', type = 'normal', origin = 'Debian', archive = 'stable-proposed-updates'):
		self.filename = path
		self.archive = archive
		self.component = 'main'
		# release version
		self.version = '1.0'
		# provider of the archive
		self.origin = origin
		# name of the distribution
		self.label = 'Debian'
		# apt will not consider taking packages unless told explicitly to do so
		self.not_automatic = 0
		if type == 'normal':
			self.index_type = PackageFileAdapter.TYPE_PACKAGE_FILE
		elif type == 'dpkg':
			self.index_type = PackageFileAdapter.TYPE_DPKG_STATUS
			self.filename = '/var/lib/dpkg/status'
			self.archive = 'now'
			self.component = ''
			self.version = ''
			self.origin = ''
			self.label = ''
		else:
			raise ValueError('Unknown type "%s"' % type)
	def __str__(self):
		return '<FakePackageFile(%s) path=%s a=%s c=%s v=%s o=%s l=%s %s>' % (self.index_type, self.filename, self.archive, self.component, self.version, self.origin, self.label, self.not_automatic)



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.status import Status
from apt_forktracer.version_adapter import VersionAdapter

class TestInstantiation(test_helper.MoxTestCase):
	def setUp(self):
		super(TestInstantiation, self).setUp()
		fv1 = FakeVersion()
		fv1.append_package_file(FakePackageFile())
		installed_version = VersionAdapter(fv1)
		fv2 = FakeVersion('1.2.4')
		fv2.append_package_file(FakePackageFile())
		candidate_version = VersionAdapter(fv2)
		self.versions_by_origin = {'Debian': [VersionAdapter(FakeVersion('foo'))], 'Another': [VersionAdapter(FakeVersion('bar')), VersionAdapter(FakeVersion('baz'))]}
		self.s = Status('foo', installed_version, candidate_version, self.versions_by_origin)
	def testCorrectness(self):
		self.assertEquals(self.s.package_name, 'foo')
		self.assertEquals(self.s.installed_version.string, '1.2.3')
		self.assertEquals(self.s.candidate_version.string, '1.2.4')
		installed_ver_pkgs = self.s.installed_version.files
		self.assertEquals(installed_ver_pkgs[0].origin, 'Debian')
		self.assertEquals(self.s.versions_from('Debian')[0].string, 'foo')
		self.assertEquals(self.s.versions_from('Another')[0].string, 'bar')
		self.assertEquals(self.s.versions_from('Another')[1].string, 'baz')
		self.assertEquals(self.s.versions_from('nowhere'), [])
		all = self.s.all_available_versions()
		self.assertEquals(len(all), 3)
		all_strings = [v.string for v in all]
		all_strings.sort()
		self.assert_('foo' in [v.string for v in all])
		self.assert_('bar' in [v.string for v in all])
	def testStringification(self):
		self.assertMatches(str(self.s), r'<Status foo .*1\.2\.3.*->.*1\.2\.4.*\[Debian: foo\]')
		self.assertMatches(str(self.s), r'<Status foo .*1\.2\.3.*->.*1\.2\.4.*\[Another: bar,baz\]')

class Test_Instantiation_Without_Official_Versions(test_helper.MoxTestCase):
	def setUp(self):
		super(Test_Instantiation_Without_Official_Versions, self).setUp()
		fv1 = FakeVersion('1.2.5')
		fv1.append_package_file(FakePackageFile())
		installed_version = VersionAdapter(fv1)
		fv2 = FakeVersion('1.2.6')
		fv2.append_package_file(FakePackageFile())
		candidate_version = VersionAdapter(fv2)
		self.versions_by_origin = {'NonDebian': [VersionAdapter(FakeVersion('foo'))]}
		self.s = Status('foo', installed_version, candidate_version, self.versions_by_origin)
	def testCorrectness(self):
		self.assertEquals(self.s.package_name, 'foo')
		self.assertEquals(self.s.installed_version.string, '1.2.5')
		self.assertEquals(self.s.candidate_version.string, '1.2.6')
		installed_ver_pkgs = self.s.installed_version.files
		self.assertEquals(installed_ver_pkgs[0].origin, 'Debian')
		self.assertEquals(self.s.versions_from('NonDebian')[0].string, 'foo')
		self.assertEquals(self.s.versions_from('nowhere'), [])
	def testStringification(self):
		self.assertMatches(str(self.s), r'<Status foo .*1\.2\.5.*->.*1\.2\.6.*')

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class Checker:
	"""Checks whether a package is non-standard enough to be reported."""
	def __init__(self, facter, verbose_mode = False):
		"""Optional last argument specifies whether this checker should run in
		verbose mode. By default it is not verbose.
		"""
		self.facter = facter
		self.verbose_mode = verbose_mode

	def check(self, package_adapter):
		"""Given a PackageAdapter object, determines whether it is sufficiently
		non-standard (i.e. non-official, or inconsistent state) to be reported.
		Returns a Status object in such case. Otherwise, returns None.

		Checks which versions are current, candidate, available and whether
		they are official. Does not do detailed version string analyses (this
		is the job of Policy, which also decides whether to ignore any
		reports).

		Currently, its algorithm is as follows:

		In default mode, returns a Status object if there is no candidate
		version, or if the candidate version is not available from an official
		source (as determined by the facter).

		In the verbose mode, additionally returns a Status object if the
		candidate and current versions are not the same (i.e. an upgrade or
		downgrade is pending).

		Raises an exception if current version is None.
		"""
		current = package_adapter.current_version
		candidate = package_adapter.candidate_version
		if current == None:
			raise ValueError('Package %s has no current version' % package_adapter)
		if candidate == None:
			return package_adapter.get_status(self.facter)
		if self.verbose_mode and candidate.string != current.string:
			return package_adapter.get_status(self.facter)
		if not candidate.is_officially_available(self.facter):
			return package_adapter.get_status(self.facter)
		return None




#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import mox
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.testlib.fake_package_file import FakePackageFile
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.depcache_adapter import DepCacheAdapter
from apt_forktracer.package_adapter import PackageAdapter,PackageAdapterFactory
from apt_forktracer.version_adapter import VersionAdapter

class Test_Package_And_Version_Reading(test_helper.MoxTestCase):
	def setUp(self):
		super(Test_Package_And_Version_Reading, self).setUp()
		self.fake = FakePackage()
		v1 = FakeVersion()
		v1.ver_str = '1.2.3'
		v1.append_package_file(FakePackageFile())
		fpf = FakePackageFile()
		fpf.not_automatic = 1
		v1.append_package_file(fpf)
		v2 = FakeVersion()
		v2.ver_str = '4.5.6'
		self.fake.version_list.append(v1)
		self.fake.version_list.append(v2)
		self.fake.current_ver = v1
		self.set_up_package_adapter_and_replay_all()
	def set_up_package_adapter_and_replay_all(self):
		self.mox.ReplayAll()
		self.p = PackageAdapter(self.fake)
	def testPackageAndVersionsReadCorrectly(self):
		self.assertEquals(self.p.name, 'afake')
		self.assertEquals(len(self.p.versions), 2)
		v1 = self.p.versions[0]
		self.assertEquals(v1.string, '1.2.3')
		self.assertEquals(len(v1.files), 2)
		v1pf0 = v1.files[0]
		self.assertEquals(v1pf0.name, '/a/fake')
		v1pf1 = v1.files[1]
		self.assert_(v1pf1.not_automatic)
		v2 = self.p.versions[1]
		self.assertEquals(v2.string, '4.5.6')
	def testStringificationWorks(self):
		s = str(self.p)
		self.assertContains(s, 'PackageAdapter')
		self.assertContains(s, ' afake ')
		self.assertMatches(s, 'v=<.*1.2.3.*->')
		vs = str(self.p.versions[0])
		self.assertContains(vs, 'VersionAdapter')
		self.assertContains(vs, '1.2.3')
		vpfs = str(self.p.versions[0].files[0])
		self.assertContains(vpfs, 'PackageFileAdapter')
		self.assertContains(vpfs, 'path=/a/fake')
		self.assertContains(vpfs, 'a=stable-proposed-updates')
		self.assertContains(vpfs, 'c=main')
		self.assertContains(vpfs, 'v=1.0')
		self.assertContains(vpfs, 'o=Debian')
		self.assertContains(vpfs, 'l=Debian')
		self.assertNotContains(vpfs, 'NONAUTO')
		vpfs2 = str(self.p.versions[0].files[1])
		self.assertContains(vpfs2, 'NONAUTO')

class Test_With_Factory_Creation(Test_Package_And_Version_Reading):
	def set_up_package_adapter_and_replay_all(self):
		self.mox.ReplayAll()
		self.p = PackageAdapterFactory().create_package_adapter(self.fake)

class Test_With_Factory_Creation_With_Candidate(Test_Package_And_Version_Reading):
	def set_up_package_adapter_and_replay_all(self):
		mock_depcache_adapter = self.mox.CreateMock(DepCacheAdapter)
		self.va = VersionAdapter(FakeVersion._create('1.2.4', []))
		mock_depcache_adapter.get_candidate_version(mox.Func(lambda pa: pa.name == 'afake')).AndReturn(self.va)
		self.mox.ReplayAll()

		self.p = PackageAdapterFactory(mock_depcache_adapter).create_package_adapter(self.fake)
	def test_candidate_version(self):
		self.assertEquals(self.p.candidate_version, self.va)
		s = str(self.p)
		self.assertMatches(s, 'v=<.*1.2.3.*->.*<.*1.2.4')

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import apt_pkg
import mox
import unittest

from apt_forktracer.checker import Checker
from apt_forktracer.policy import Policy
from apt_forktracer.reporter import Reporter
from apt_forktracer.status import Status
from apt_forktracer.depcache_adapter import DepCacheAdapter
from apt_forktracer.testlib import test_helper
from apt_forktracer.apt_pkg_adapter import AptPkgAdapter
from apt_forktracer.cache_adapter import CacheAdapterFactory
from apt_forktracer.testlib.fake_cache import FakeCache
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.package_adapter import PackageAdapterFactory
from apt_forktracer.version_adapter import VersionAdapter

class Test_Base_Cache_Adapter(test_helper.MoxTestCase):
	def setUp(self):
		super(Test_Base_Cache_Adapter, self).setUp()
		self.set_up_fake_cache()
		self.mock_policy = self.mox.CreateMock(Policy)
		self.mock_reporter = self.mox.CreateMock(Reporter)
		self.mock_depcache_adapter = self.mox.CreateMock(DepCacheAdapter)
		self.package_adapter_factory = PackageAdapterFactory(self.mock_depcache_adapter)
		self.set_up_apt_pkg()
		self.apt_pkg_adapter = AptPkgAdapter(self.fake_apt_pkg_module)
		self.ca = CacheAdapterFactory().create_cache_adapter(self.fake_cache, self.apt_pkg_adapter, self.mock_reporter)
		self.mock_checker = self.mox.CreateMock(Checker)
		self.version_adapter = self.mox.CreateMock(VersionAdapter)
		self.version_adapter.string = '1.2.3'
		self.mock_status = self.mox.CreateMock(Status)
	def set_up_fake_cache(self):
		self.fake_cache = FakeCache()
		self.set_up_fake_cache_tweak()
	def set_up_fake_cache_tweak(self):
		pass
	def set_up_apt_pkg(self):
		self.fake_apt_pkg_module = self._create_mock_apt_pkg_module()
		test_helper.copy_state_constants(self.fake_apt_pkg_module, apt_pkg)
	def test_basic_stringification_works(self):
		self.mox.ReplayAll()
		self.assertContains(str(self.ca), 'CacheAdapter')
	def test_states_copied(self):
		self.mox.ReplayAll()
		self.assertEquals(self.ca.states_we_check[0], apt_pkg.CURSTATE_INSTALLED)
		self.assertEquals(self.ca.states_we_check[1], apt_pkg.CURSTATE_HALF_CONFIGURED)
		self.assertEquals(self.ca.states_we_check[2], apt_pkg.CURSTATE_HALF_INSTALLED)
		self.assertEquals(self.ca.states_we_check[3], apt_pkg.CURSTATE_UNPACKED)

class Test_Empty_Cache_Adapter(Test_Base_Cache_Adapter):
	def test_stringification_with_empty_cache(self):
		self.mox.ReplayAll()
		self.assertContains(str(self.ca), '0 package(s)')
	def test_invokes_checker_zero_times_with_empty_cache(self):
		self.mox.ReplayAll()
		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_One_Installed_Package_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage())
	def test_stringification_with_one_package(self):
		self.mox.ReplayAll()
		self.assertContains(str(self.ca), '1 package(s)')
	def test_invokes_checker_one_time_with_cache_containing_one_package(self):
		self.mock_depcache_adapter.get_candidate_version(mox.Func(lambda o: o.name == 'afake')).AndReturn(self.version_adapter)
		self.mock_checker.check(mox.IgnoreArg()).AndReturn(self.mock_status)
		self.mock_policy.should_report(self.mock_status).AndReturn(True)
		self.mock_reporter.report(self.mock_status)
		self.mox.ReplayAll()

		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_Two_Installed_Packages_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage(name = 'foo'))
		self.fake_cache.append_package(FakePackage(name = 'foo'))
	def test_invokes_checker_two_times_with_cache_containing_one_package(self):
		self.mock_depcache_adapter.get_candidate_version(mox.Func(lambda o: o.name == 'foo')).AndReturn(self.version_adapter)
		self.mock_depcache_adapter.get_candidate_version(mox.Func(lambda o: o.name == 'foo')).AndReturn(self.version_adapter)
		self.mock_checker.check(mox.IgnoreArg()).AndReturn(self.mock_status)
		self.mock_checker.check(mox.IgnoreArg()).AndReturn(self.mock_status)
		self.mock_policy.should_report(self.mock_status).AndReturn(True)
		self.mock_policy.should_report(self.mock_status).AndReturn(True)
		self.mock_reporter.report(self.mock_status)
		self.mock_reporter.report(self.mock_status)
		self.mox.ReplayAll()

		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_One_Not_Installed_Package_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage(current_state = apt_pkg.CURSTATE_NOT_INSTALLED))
	def test_invokes_checker_zero_times_with_cache_containing_one_not_installed_package(self):
		self.mox.ReplayAll()
		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_One_Conffiles_Package_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage(current_state = apt_pkg.CURSTATE_CONFIG_FILES))
	def test_invokes_checker_zero_times_with_cache_containing_one_package_with_just_config_files(self):
		self.mox.ReplayAll()
		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_One_Half_Configured_Package_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage(current_state = apt_pkg.CURSTATE_HALF_CONFIGURED))
	def test_invokes_checker_one_time_with_cache_containing_one_package_half_configured(self):
		self.mock_depcache_adapter.get_candidate_version(mox.Func(lambda o: o.name == 'afake')).AndReturn(self.version_adapter)
		self.mock_checker.check(mox.IgnoreArg()).AndReturn(self.mock_status)
		self.mock_policy.should_report(self.mock_status).AndReturn(True)
		self.mock_reporter.report(self.mock_status)
		self.mox.ReplayAll()
		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_One_Half_Installed_Package_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage(current_state = apt_pkg.CURSTATE_HALF_INSTALLED))
	def test_invokes_checker_one_time_with_cache_containing_one_package_half_installed(self):
		self.mock_depcache_adapter.get_candidate_version(mox.Func(lambda o: o.name == 'afake')).AndReturn(self.version_adapter)
		self.mock_checker.check(mox.IgnoreArg()).AndReturn(None)
		self.mox.ReplayAll()
		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

class Test_One_Unpacked_Package_Cache_Adapter(Test_Base_Cache_Adapter):
	def set_up_fake_cache_tweak(self):
		self.fake_cache.append_package(FakePackage(current_state = apt_pkg.CURSTATE_UNPACKED))
	def test_invokes_checker_one_time_with_cache_containing_one_package_half_configured(self):
		self.mock_depcache_adapter.get_candidate_version(mox.Func(lambda o: o.name == 'afake')).AndReturn(self.version_adapter)
		self.mock_checker.check(mox.Func(lambda o:o.name == 'afake' and o.candidate_version.string == '1.2.3')).AndReturn(self.mock_status)
		self.mock_policy.should_report(self.mock_status).AndReturn(False)
		self.mox.ReplayAll()
		self.ca.run(self.mock_checker, self.mock_policy, self.package_adapter_factory)

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class Status:
	"""Models a potentially interesting piece of information about the status
	   of an installed version of a package, WRT other available versions.
	   """
	def __init__(self, package_name, installed_version, candidate_version, versions_by_origin = {}):
		self.package_name = package_name
		self.installed_version = installed_version
		self.candidate_version = candidate_version
		self.versions_by_origin = versions_by_origin

	def versions_from(self, origin):
		"""Returns a list of versions available from a given origin."""
		if self.versions_by_origin.has_key(origin):
			return self.versions_by_origin[origin]
		else:
			return []

	def all_available_versions(self):
		"""Returns a list of all versions available from any source.
		Does not include the installed or candidate versions, if they are not
		available from some source."""
		ret = []
		for a in self.versions_by_origin.values():
			ret.extend(a)
		return ret

	def __str__(self):
		version_info = ''
		for origin,versions in self.versions_by_origin.items():
			version_info += '[%s: %s]' % (origin, ','.join([v.string for v in versions]))
		return '<Status %s %s->%s %s>' % (self.package_name, self.installed_version, self.candidate_version, version_info)
	__repr__ = __str__



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import mox
import re

from apt_forktracer.facter import Facter

def copy_state_constants(to_obj, from_obj):
	to_obj.CURSTATE_INSTALLED = from_obj.CURSTATE_INSTALLED
	to_obj.CURSTATE_HALF_CONFIGURED = from_obj.CURSTATE_HALF_CONFIGURED
	to_obj.CURSTATE_HALF_INSTALLED = from_obj.CURSTATE_HALF_INSTALLED
	to_obj.CURSTATE_UNPACKED = from_obj.CURSTATE_UNPACKED

class Advanced_Version_Comparator:
	def __init__(self):
		# tuple: v1, v2, relation
		# relation == 1 iff v1 << v2
		# == 0  iff v1 == v2
		# == -1 iff v1 >> v2
		self.comparisons = [
			('1', '0', 1),
			('1', '1', 0),
			('0', '1', -1),

			('0.1-1', '0.1-1', 0),
			('0.1-1', '0.1-2', -1),
			('0.1-2', '0.1-1~sl1', 1),
			('0.1-1', '0.1-1~sl1', 1),
			('2.6.1-2etch1', '2.6.1-2etch2', -1),
			('2.6.1-2etch1', '2.6.1-2etch1', 0),
			('2.6.1-2etch2', '2.6.1-2etch2', 0),
			('1:1.5.6.3-1.1ubuntu2~mowsiany.1', '1:1.5.6.3-1.1ubuntu2~mowsiany.1', 0),
			('0.5-5~sl1', '0.5-5~sl1', 0),
			('0.5-5', '0.5-5~sl1', 1),
			('0.5-5', '0.5-5', 0),
			('1:1.5.2.5-2build1', '1:1.5.2.5-2build1', 0),
			('1:1.5.2.5-2build1', '1:1.5.6.3-1.1ubuntu2~mowsiany.1', -1),
			('1:1.5.2.5-2build1', '1:1.5.6.3-1.1ubuntu2', -1),
	
			('1.9', '2.0', -1),
	
			('2.0~1', '0', 1),
			('2.0~1', '1', 1),
			('2.0~1', '2.0~1', 0),
			('2.0~1', '2.0', -1),
	
			('2.0', '0', 1),
			('2.0', '2.0~1', 1),
			('2.0', '2.0~1', 1),
			('2.0', '2.0', 0),
			('2.0', '2.1~1', -1),
			('2.0', '2.1~2', -1),
			('2.0', '2.1', -1),
			('2.0', '2.2', -1),
	
			('2.1~1', '2.1~1', 0),
			('2.1~1', '2.1', -1),
	
			('2.1~2', '2.1~2', 0),
			('2.1~2', '2.1', -1),
	
			('2.1', '2.0~1', 1),
			('2.1', '2.0', 1),
			('2.1', '2.1~1', 1),
			('2.1', '2.1~2', 1),
			('2.1', '2.1', 0),
			('2.1', '2.2~1', -1),
			('2.1', '2.2~2', -1),
			('2.1', '2.2', -1),
	
			('2.2~2', '2.2~2', 0),
			('2.2~2', '2.2', -1),
	
			('2.2', '2.1~1', 1),
			('2.2', '2.1~2', 1),
			('2.2', '2.1', 1),
			('2.2', '2.2~1', 1),
			('2.2', '2.2', 0),
		]
	def compare(self, v1, v2):
		if v1 == v2:
			return 0
		for comparison in self.comparisons:
			if v1 == comparison[0] and v2 == comparison[1]:
				return comparison[2]
		for comparison in self.comparisons:
			if v1 == comparison[1] and v2 == comparison[0]:
				return 0 - comparison[2]
		if v1.find('~') >= 0 or v2.find('~') >= 0:
			raise ValueError('not attempting to compare %s with %s (tilde found)' % (v1, v2))
		return cmp(v1, v2)


class MoxTestCase(mox.MoxTestBase):
	def assertContains(self, haystack, needle):
		self.assert_(haystack.find(needle) >= 0, 'could not find %s in %s' % (needle, haystack))
	def assertNotContains(self, haystack, needle):
		self.assert_(haystack.find(needle) < 0, 'found "%s" in "%s"' % (needle, haystack))
	def assertMatches(self, haystack, regex):
		self.assert_(re.compile(regex).search(haystack), 'could not match "%s" against "%s"' % (haystack, regex))

	def assertRaisesWithMessageContaining(self, exception_class, message_snippet, method, *args, **kwargs):
		succeeded = False
		try:
			method(*args, **kwargs)
			succeeded = True
		except exception_class, e:
			self.assertContains(e.message, message_snippet)
		except Exception, e:
			self.fail('%s failed with %s(%s) instead of %s' % (method, type(e), e.message, exception_class))
		if succeeded:
			self.fail('%s did not fail with %s' % (method, exception_class))

	def _create_mock_facter(self, id):
		mock_facter = self.mox.CreateMock(Facter)
		mock_facter.distributors_id = id
		return mock_facter

	def struct(self, **kwargs):
		class Struct:
			def __init__(self, **entries): self.__dict__.update(entries)
		return Struct(**kwargs)

	def _create_mock_apt_pkg_module(self):
		mock_apt_pkg_module = self.mox.CreateMockAnything()
		comparator = Advanced_Version_Comparator()
		mock_apt_pkg_module.version_compare = lambda x, y: comparator.compare(x, y)
		mock_apt_pkg_module.init_config = lambda: None
		mock_apt_pkg_module.init_system = lambda: None
		return mock_apt_pkg_module




#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.policy import VerbosePolicy

class TestVerbosePolicy(unittest.TestCase):
	def setUp(self):
		self.policy = VerbosePolicy()
	def testTrue(self):
		self.assert_(self.policy.should_report(None))

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import mox
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.config_parser import ConfigParser
from apt_forktracer.config import Config

class Test_ConfigParser(test_helper.MoxTestCase):
	def setUp(self):
		super(Test_ConfigParser, self).setUp()
		self.cp = ConfigParser()
		self.c = self.mox.CreateMock(Config)
	def test_loading_empty_file(self):
		self.mox.ReplayAll()
		self.assertEquals(self.cp.load([], self.c), [])
	def test_loading_invalid_syntax_file(self):
		fake_file = ['\n', 'invalid line\n']
		self.mox.ReplayAll()
		self.assertRaisesWithMessageContaining(Exception, 'line 2', self.cp.load, fake_file)
	def test_loading_invalid_stanza_file(self):
		fake_file = ['\n', 'package:val\n', 'acCepT-OrigiN: val2\n']
		self.mox.ReplayAll()
		self.assertRaisesWithMessageContaining(Exception, 'line 3', self.cp.load, fake_file)
	def test_loading_valid_file(self):
		fake_file = ['\n', 'package:val\n', 'acCepT-OrigiN: val2\n', ' track-oriGin : a  spaced val \n', 'track-version:foo']
		self.c.add(mox.Func(lambda stanza: stanza.get('package') == 'val'))
		self.mox.ReplayAll()

		ret = self.cp.load(fake_file, self.c)
		self.assertEquals(len(ret), 1)
		self.assertEquals(ret[0].get('package'), 'val')

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.testlib.fake_package import FakePackage
from apt_forktracer.testlib.fake_version import FakeVersion
from apt_forktracer.version_adapter import VersionAdapter
from apt_forktracer.version_checker import VersionChecker

class VersionCheckerTest(test_helper.MoxTestCase):
	def setUp(self):
		super(VersionCheckerTest, self).setUp()
		self.fp = FakePackage()
		self.facter = self._create_mock_facter('Debian')
		self.setUpChecker()
	def setUpChecker(self):
		self.vchecker = VersionChecker(self.facter)
	def test_analyze_version_without_sources_returns_true(self):
		v = VersionAdapter(FakeVersion._create('1.2.3', []))
		self.assert_(self.vchecker.analyze(v))
	def test_analyze_version_only_in_dpkg_status_file_returns_true(self):
		v = VersionAdapter(FakeVersion._create('1.2.3', ['dpkg']))
		self.assert_(self.vchecker.analyze(v))
	def test_analyze_version_in_dpkg_status_file_and_unofficial_source_returns_true(self):
		v = VersionAdapter(FakeVersion._create('1.2.3', ['dpkg', 'NonDebian']))
		self.assert_(self.vchecker.analyze(v))
	def test_analyze_version_in_dpkg_status_file_and_official_source_returns_false(self):
		v = VersionAdapter(FakeVersion._create('1.2.3', ['dpkg', 'Debian']))
		self.assert_(not self.vchecker.analyze(v))
	def test_analyze_version_in_dpkg_status_file_and_both_official_and_unofficial_source_returns_false(self):
		v = VersionAdapter(FakeVersion._create('1.2.3', ['dpkg', 'Debian', 'NonDebian']))
		self.assert_(not self.vchecker.analyze(v))

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import apt_pkg
import getopt

from apt_forktracer.apt_pkg_adapter import AptPkgAdapter, NullProgress
from apt_forktracer.cache_adapter import CacheAdapterFactory
from apt_forktracer.checker import Checker
from apt_forktracer.config import Config
from apt_forktracer.config_finder import ConfigFinder
from apt_forktracer.config_parser import ConfigParser
from apt_forktracer.depcache_adapter import DepCacheAdapterFactory
from apt_forktracer.facter import Facter
from apt_forktracer.package_adapter import PackageAdapterFactory
from apt_forktracer.policy import Policy, VerbosePolicy
from apt_forktracer.reporter import Reporter

def run(verbose):
	facter = Facter()
	reporter = Reporter()
	apt_pkg_adapter = AptPkgAdapter(apt_pkg)
	apt_pkg_adapter.init()
	cache_adapter = apt_pkg_adapter.get_cache_adapter(CacheAdapterFactory(), reporter, NullProgress())
	apt_depcache_adapter = apt_pkg_adapter.get_depcache_adapter(DepCacheAdapterFactory())
	package_adapter_factory = PackageAdapterFactory(apt_depcache_adapter)
	checker = Checker(facter, verbose)
	config = Config()
	config_finder = ConfigFinder('/etc/apt/forktracer.conf', '/etc/apt/forktracer.d')
	config_parser = ConfigParser()
	for path, file in config_finder:
		config_parser.load(file, config)
	if verbose:
		policy = VerbosePolicy()
	else:
		policy = Policy(apt_pkg_adapter, facter, config)
	cache_adapter.run(checker, policy, package_adapter_factory)

def main(sys):
	try:
		opts, args = getopt.getopt(sys.argv[1:], 'v', ['verbose'])
	except getopt.GetoptError, err:
		print str(err)
		sys.exit(1)
	verbose = False
	for o, a in opts:
		if o in ('-v', '--verbose'):
			verbose = True
		else:
			assert False, 'unhandled option'
	run(verbose)




# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from apt_forktracer.config_stanza import ConfigStanza

class ConfigParser:
	"""Parses a text file, producing ConfigStanza objects."""

	def __add(self, stanza, lineno, ret, config):
		"""Private method for gathering stanzas."""	
		if not stanza.is_empty():
			ret.append(stanza.finish(lineno))
			if config:
				config.add(stanza)
			stanza = ConfigStanza()
		return stanza

	def load(self, file, config = None):
		"""Parses an open file object, and returns a list of stanzas found. If
		an optional config argument (a Config object) is provided, it also adds
		the stanzas to it.
		
		Parsing errors are raised as exceptions.
		"""
		ret = []
		stanza = ConfigStanza()
		lineno = 0
		for line in file:
			lineno += 1
			line = line.strip()
			if line == '':
				stanza = self.__add(stanza, lineno, ret, config)
				continue
			colon = line.find(':')
			if colon != -1:
				key, val = line.split(':', 1)
				stanza.set(key.strip(), val.strip(), lineno)
			else:
				raise ValueError('invalid line %d: %s' % (lineno, repr(line)))
		stanza = self.__add(stanza, lineno, ret, config)
		return ret



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008-2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import re
import subprocess
import sys

class Facter:
	"""Reports some facts about the system. Currently just the distributor's id
	(available through the distributors_id attribute).

	Tries to import the lsb_release Python module, which should be fast and
	provide most correct information.

	If that fails, tries to run lsb_release(1) and read its output, which
	should be functionally equivalent, but not as fast, since it requires a
	fork/exec.

	If that fails too, reads /etc/lsb-release directly as the last resort. This
	might not be correct in all cases, as strictly speaking that file is not
	required, but just provides a way to override values detected from the
	system.

	Throws an exception if all methods fail.

	The optional attributes are mostly for testing purposes.
	"""
	def __init__(self, lsb_release_module = 'lsb_release', lsb_release = 'lsb_release --id --short', file = '/etc/lsb-release'):
		self.distributors_id = self.get_distrib_id_from_module(lsb_release_module)
		if not self.distributors_id:
			self.distributors_id = self.get_distrib_id_from_command(lsb_release)
		if not self.distributors_id:
			self.distributors_id = self.get_distrib_id_from_file(file)
		if not self.distributors_id:
			raise RuntimeError('Could not obtain distributors id from lsb_release')

	def get_distrib_id_from_file(self, file_name):
		p = re.compile(r'DISTRIB_ID\s*=\s*(\S+)')
		try:
			f = open(file_name, 'r')
			for line in f:
				m = p.search(line)
				if m:
					f.close()
					return m.group(1)
		except IOError:
			pass

	def get_distrib_id_from_command(self, command):
		try:
			p = subprocess.Popen(command, stdout = subprocess.PIPE, shell = True)
			output = p.stdout.readline()
			p.stdout.close()
			p.wait()
			if p.returncode == 0:
				return output.rstrip('\n')
		except OSError:
			pass

	def get_distrib_id_from_module(self, module_name):
		try:
			__import__(module_name)
			# we need to look up in sys.modules as __import__(name) only returns top-level module if name contains dots
			module = sys.modules[module_name]
			distinfo = module.get_distro_information()
			return distinfo['ID']
		except ImportError:
			pass
 



#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from distutils.core import setup
setup(name = 'apt-forktracer',
	version = '0.3',
	description = 'Debian packages version tracing utility',
	author = 'Marcin Owsiany',
	author_email = 'marcin@owsiany.pl',
	packages = ['apt_forktracer'],
	package_dir = {'': 'lib'},
	scripts = ['apt-forktracer']
	)




# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class NullProgress:
	"""Makes libapt shut up when passed to Cache()."""
	def __init__(self, *args, **kwargs):
		pass
	def update(self, *args, **kwargs):
		pass
	def done(self, *args, **kwargs):
		pass

class AptPkgAdapter:
	"""Wraps the apt_pkg module.
	
	The reason for all the *Adapter classes is because the APT API is declared
	as prone to change. By encapsulating access to the APT API in the adapter
	classes, we decouple the rest of the codebase from it, hopefully making it
	easier to port to potential newer incompatible libapt versions.
	"""
	def __init__(self, apt_pkg):
		"""apt_pkg is the imported libapt module to wrap."""
		self.apt_pkg = apt_pkg
		self.state_installed = apt_pkg.CURSTATE_INSTALLED
		self.state_half_installed = apt_pkg.CURSTATE_HALF_INSTALLED
		self.state_half_configured = apt_pkg.CURSTATE_HALF_CONFIGURED
		self.state_unpacked = apt_pkg.CURSTATE_UNPACKED
		self.apt_cache = None
		self.inited = False

	def init(self):
		"""Initializes libapt. Must be called before any other method."""
		self.apt_pkg.init_config()
		self.apt_pkg.init_system()
		self.inited = True

	def _assert_initialized(self):
		"""Helper method to check whether init() had been called."""
		if not self.inited:
			raise Exception('not initialized')

	def get_cache_adapter(self, cache_adapter_factory, reporter, progress):
		"""Returns a CacheAdapter object, given a CacheAdapterFactory, a
		Reporter and libapt progress reporting object.

		Throws an exception if called before init().
		"""
		self._assert_initialized()
		self.apt_cache = self.apt_pkg.Cache(progress)
		return cache_adapter_factory.create_cache_adapter(self.apt_cache, self, reporter)

	def get_depcache_adapter(self, depcache_adapter_factory):
		"""Returns a DepCacheAdapter object, given a DepCacheAdapterFactory.
		Must be called after get_cache_adapter() was invoked.

		Throws an exception if called before init(). 
		"""
		self._assert_initialized()
		if not self.apt_cache:
			raise Exception('you must call get_cache_adapter() earlier')
		apt_depcache = self.apt_pkg.DepCache(self.apt_cache)
		return depcache_adapter_factory.create_depcache_adapter(apt_depcache)

	def version_compare(self, version_a, version_b):
		"""Returns -1, 0 or 1 depending on whether version_a string is considered
		older, equal to, or newer than version_b string.
		
		Throws an exception if either version equals None.
		Throws an exception if called before init(). 
		"""
		self._assert_initialized()
		if version_a == None or version_b == None:
			raise ValueError('cannot compare a None version string')
		return self.apt_pkg.version_compare(version_a, version_b)

	def version_sort(self, versions):
		"""Sorts the given versions list of VersionAdapters in situ and also
		returns it. Sorted list is in newest to oldest order.

		Throws an exception if called before init(). 
		"""
		self._assert_initialized()
		versions.sort(lambda va,vb: self.version_compare(va.string, vb.string), reverse = True)
		return versions

	def version_max(self, versions):
		"""Returns the newest VersionAdapter object in the versions list, or
		None, if versions is empty.

		Throws an exception if called before init(). 
		"""
		self._assert_initialized()
		max = None
		for v in versions:
			if max == None or self.version_compare(v.string, max.string) > 0:
				max = v
		return max




#!/usr/bin/python
# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import unittest

from apt_forktracer.testlib import test_helper
from apt_forktracer.config_stanza import ConfigStanza

class Test_ConfigStanza(test_helper.MoxTestCase):
	def setUp(self):
		super(Test_ConfigStanza, self).setUp()
		self.cs = ConfigStanza()
		self.assert_(self.cs.is_empty())
	def test_empty(self):
		self.assertRaisesWithMessageContaining(ValueError, 'line 2', self.cs.finish, 2)
	def test_all_attributes(self):
		self.cs.set('Package', 'dpkg', 2)
		self.assert_(not self.cs.is_empty())
		self.cs.set('Accept-Origin', 'NonDebian Stuff', 3)
		self.cs.set('Track-oriGIN', 'Debian', 4)
		self.cs.set('track-version', '1.2.3', 5)
		self.assertEquals(self.cs.get('package'), 'dpkg')
		self.assertEquals(self.cs.get('accept-origin'), 'NonDebian Stuff')
		self.assert_(self.cs.matches('accept-origin', 'NonDebian Stuff'))
		self.assert_(not self.cs.matches('accept-origin', 'Debian'))
		self.assertEquals(self.cs.get('track-origin'), 'Debian')
		self.assertEquals(self.cs.get('track-version'), '1.2.3')
		self.assert_(not self.cs.is_empty())
		self.assertEquals(self.cs.finish(6), self.cs)
	def test_invalid_attribute(self):
		self.assertRaisesWithMessageContaining(ValueError, 'invalid tag', self.cs.set, 'FoO', 'bar', 2)
		self.assertRaisesWithMessageContaining(ValueError, 'line 2', self.cs.set, 'FoO', 'bar', 2)
	def test_wildcard_attributes(self):
		self.cs.set('Package', 'dpkg', 2)
		self.assert_(not self.cs.is_empty())
		self.cs.set('Accept-Origin', '*', 3)
		self.cs.set('Track-oriGIN', '*', 4)
		self.cs.set('track-version', '1.2.3', 5)
		self.assertEquals(self.cs.get('accept-origin'), '*')
		self.assert_(self.cs.matches('accept-origin', 'whatever'))
		self.assertEquals(self.cs.get('track-origin'), '*')
		self.assert_(self.cs.matches('track-origin', 'whatever'))
		self.assert_(not self.cs.is_empty())
		self.assertEquals(self.cs.finish(6), self.cs)

if __name__ == '__main__':
	unittest.main()



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class PackageFileAdapter:
	"""Wrapper for the aptCache::PackageFile libapt class. See AptPkgAdapter
	for an explanation of why this is needed."""
	TYPE_PACKAGE_FILE = 'Debian Package Index'
	TYPE_DPKG_STATUS = 'Debian dpkg status file'
	def __init__(self, apt_package_file):
		self.name = apt_package_file.filename
		self.archive = apt_package_file.archive
		self.component = apt_package_file.component
		self.version = apt_package_file.version
		self.origin = apt_package_file.origin
		self.label = apt_package_file.label
		self.not_automatic = apt_package_file.not_automatic
		self.index_type = apt_package_file.index_type

	def is_official(self, facter):
		"""Returns True if the package file's origin matches distributor's ID
		according to facter."""
		if self.index_type != PackageFileAdapter.TYPE_PACKAGE_FILE:
			return False
		else:
			return self.origin == facter.distributors_id

	def __str__(self):
		"""Returns a string which includes some attributes of the object."""
		if self.index_type == PackageFileAdapter.TYPE_DPKG_STATUS:
			return '<PackageFileAdapter(dpkg status) path=%s>' % (self.name)
		auto = ''
		if self.not_automatic:
			auto = ' NONAUTO'
		return '<PackageFileAdapter path=%s a=%s c=%s v=%s o=%s l=%s%s>' % (self.name, self.archive, self.component, self.version, self.origin, self.label, auto)
	__repr__ = __str__



# apt-forktracer - a utility for managing package versions
# Copyright (C) 2008,2010 Marcin Owsiany <porridge@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from apt_forktracer.package_file_adapter import PackageFileAdapter

class VersionAdapter:
	"""Mirror of the aptCache::Version libapt class. See AptPkgAdapter for an
	explanation of why this is needed."""
	def __init__(self, apt_version):
		self.string = apt_version.ver_str
		self.files = [PackageFileAdapter(pf[0]) for pf in apt_version.file_list]

	def is_officially_available(self, facter):
		"""Returns True if any of the files considers itself official."""
		for file in self.files:
			if file.is_official(facter):
				return True
		return False

	def __str__(self):
		files = ''
		for f in self.files:
			files += str(f) + ','
		return '<VersionAdapter %s [%s]>' % (self.string, files)
	__repr__ = __str__
